// ***************************************************************************
// ***************************************************************************
// ***************************************************************************
// ***************************************************************************

#include <stdio.h>
#include "platform.h"
#include "xbasic_types.h"
#include "xstatus.h"
#include "xil_io.h"
#include "xparameters.h"
#include "xuartlite_l.h"

#define CF_BASEADDR   XPAR_AXI_AD9649_0_BASEADDR
#define DDR_BASEADDR  XPAR_DDR3_SDRAM_S_AXI_BASEADDR
#define DMA_BASEADDR  XPAR_AXI_DMA_0_BASEADDR
#define UART_BASEADDR XPAR_RS232_UART_1_BASEADDR
#define SPI_BASEADDR  XPAR_AXI_SPI_0_BASEADDR

// ***************************************************************************
// ***************************************************************************

extern char inbyte(void);
void xil_printf(const char *ctrl1, ...);

void delay_ms(u32 ms_count) {
  u32 count;
  for (count = 0; count < ((ms_count * 100000) + 1); count++) {
    asm("nop");
  }
}

void ad9649_spi_write(u32 addr, u32 data) {
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x1e6); // control register (rst fifos)
  Xil_Out32((SPI_BASEADDR + 0x70), 0x002); // chip select
  Xil_Out32((SPI_BASEADDR + 0x68), ((addr>>8) & 0xff)); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), (addr & 0xff)); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), data); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x180); // control register
}

u32 ad9649_spi_read(u32 addr) {
  u32 rdata;
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x1e6); // control register (rst fifos)
  Xil_Out32((SPI_BASEADDR + 0x70), 0x002); // chip select
  Xil_Out32((SPI_BASEADDR + 0x68), (0x80 | ((addr>>8) & 0xff))); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), (addr & 0xff)); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), 0x000); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  rdata = Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x180); // control register
  return(rdata);
}

void ad9517_spi_write(u32 addr, u32 data) {
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x1e6); // control register (rst fifos)
  Xil_Out32((SPI_BASEADDR + 0x70), 0x001); // chip select
  Xil_Out32((SPI_BASEADDR + 0x68), ((addr>>8) & 0xff)); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), (addr & 0xff)); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), data); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x180); // control register
}

u32 ad9517_spi_read(u32 addr) {
  u32 rdata;
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x1e6); // control register (rst fifos)
  Xil_Out32((SPI_BASEADDR + 0x70), 0x001); // chip select
  Xil_Out32((SPI_BASEADDR + 0x68), (0x80 | ((addr>>8) & 0xff))); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), (addr & 0xff)); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), 0x000); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  rdata = Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x180); // control register
  return(rdata);
}

void adc_capture(u32 qwcnt, u32 sa) {
  Xil_Out32((DMA_BASEADDR + 0x030), 0); // clear dma operations
  Xil_Out32((DMA_BASEADDR + 0x030), 1); // enable dma operations
  Xil_Out32((DMA_BASEADDR + 0x048), sa); // capture start address
  Xil_Out32((DMA_BASEADDR + 0x058), (qwcnt * 8)); // number of bytes
  Xil_Out32((CF_BASEADDR + 0x00c), 0x0); // capture disable
  Xil_Out32((CF_BASEADDR + 0x010), 0xf); // clear status
  Xil_Out32((CF_BASEADDR + 0x014), 0xf); // clear status
  Xil_Out32((CF_BASEADDR + 0x00c), (0x10000 | (qwcnt - 1)));
  do {delay_ms(1);}
  while ((Xil_In32(CF_BASEADDR + 0x010) & 0x1) == 1);
  if ((Xil_In32(CF_BASEADDR + 0x010) & 0x02) == 0x02) {
    xil_printf("overflow occured, capture may be corrupted\n\r");
  }
  microblaze_flush_dcache();
  microblaze_invalidate_dcache();
}

void adc_test(u32 mode, u32 format) {
  u32 n;
  u32 rdata;
  u32 edata;
  edata = 0;
  ad9649_spi_write(0x14, format);
  ad9649_spi_write(0x0d, mode);
  ad9649_spi_write(0xff, 0x01);
  adc_capture(16, DDR_BASEADDR);
  xil_printf("adc_test(): mode(%2d), format(%2d)\n\r", mode, format);
  if ((mode == 0x5) || (mode == 0x6)) {
    if (format == 0x1) return;
    Xil_Out32((CF_BASEADDR+0x24), ((mode == 0x5) ? 0x01 : 0x00));
    delay_ms(10);
    Xil_Out32((CF_BASEADDR+0x14), 0xffff);
    delay_ms(100);
    if ((Xil_In32(CF_BASEADDR+0x14) & 0xffff) != 0) {
      xil_printf("  ERROR: PN status(%04x).\n\r", Xil_In32(CF_BASEADDR+0x14));
    }
    return;
  }
  for (n = 0; n < 32; n++) {
    rdata = Xil_In32(DDR_BASEADDR+(n*4));
    if ((mode == 0x1) && (format == 0)) edata = 0x20002000;
    if ((mode == 0x2) && (format == 0)) edata = 0x3fff3fff;
    if ((mode == 0x3) && (format == 0)) edata = 0x00000000;
    if ((mode == 0x1) && (format == 1)) edata = 0x00000000;
    if ((mode == 0x2) && (format == 1)) edata = 0x1fff1fff;
    if ((mode == 0x3) && (format == 1)) edata = 0x20002000;
    if (((mode == 0x4) || (mode == 0x7)) && (n == 0)) edata = (rdata & 0xffff);
    if ((mode == 0x4) && (n == 0)) edata = (edata == 0x2aaa) ? 0x15552aaa : 0x2aaa1555;
    if ((mode == 0x7) && (n == 0)) edata = (edata == 0x3fff) ? 0x00003fff : 0x3fff0000;
    if (rdata != edata) {
      xil_printf("  ERROR[%2d]: rcv(%08x), exp(%08x)\n\r", n, rdata, edata);
    }
  }
}

u32 adc_delay_1(u32 delay) {

  u32 i;
  u32 rdata;

  for (i = 0; i < 8; i++) {
    Xil_Out32((CF_BASEADDR + 0x01c), 0x00000); // delay write
    Xil_Out32((CF_BASEADDR + 0x01c), (0x20000 | (i<<8) | delay)); // delay write
    Xil_Out32((CF_BASEADDR + 0x01c), (0x20000 | (i<<8) | delay)); // delay write
  }
  for (i = 0; i < 8; i++) {
    Xil_Out32((CF_BASEADDR + 0x01c), 0x00000); // delay read
    Xil_Out32((CF_BASEADDR + 0x01c), (0x30000 | (i<<8))); // delay read
    Xil_Out32((CF_BASEADDR + 0x01c), (0x30000 | (i<<8))); // delay read
    rdata = Xil_In32(CF_BASEADDR + 0x020);
    if (rdata != (0x100 | delay)) {
      xil_printf("adc_delay_1: sel(%2d), data(%04x)\n\r", i, rdata);
    }
  }
  return(0);
}

u32 adc_delay() {

  u32 delay;
  u32 error;
  u32 lasterr;
  u32 startdelay;
  u32 stopdelay;

  lasterr = 1;
  startdelay = 32;
  stopdelay = 31;

  for (delay = 0; delay < 32; delay++) {
    adc_delay_1(delay);
    delay_ms(10);
    Xil_Out32((CF_BASEADDR + 0x14), 0xff);
    delay_ms(100);
    error = Xil_In32(CF_BASEADDR + 0x14) & 0xff;
    if (error == 0) {
      if (lasterr == 1) {
        startdelay = delay;
      }
      lasterr = 0;
    } else {
      if (lasterr == 0) {
        stopdelay = delay;
        break;
      }
      lasterr = 1;
    }
  }
  if (startdelay > 31) {
    adc_delay_1(0);
    return(1);
  }
  delay = startdelay + ((stopdelay - startdelay)/2);
  xil_printf("adc_setup: setting zero error delay (%d)\n\r", delay);
  adc_delay_1(delay);
  return(0);
}

void adc_setup(u32 dco_delay) {
  Xil_Out32((CF_BASEADDR + 0x01c), 0x00000);
  Xil_Out32((CF_BASEADDR + 0x01c), 0x2ff00);
  Xil_Out32((CF_BASEADDR + 0x01c), 0x2ff00);
  ad9649_spi_write(0x0d, 0x05);
  ad9649_spi_write(0x14, 0x08);
  ad9649_spi_write(0xff, 0x01);
  ad9649_spi_write(0xff, 0x00);
  Xil_Out32((CF_BASEADDR + 0x24), 0x1);
  ad9649_spi_write(0x16, 0x00);
  ad9649_spi_write(0xff, 0x01);
  ad9649_spi_write(0xff, 0x00);
  xil_printf("AD9649[0x016]: %02x\n\r", ad9649_spi_read(0x16));
  delay_ms(10);
  if (adc_delay()) {
    ad9649_spi_write(0x16, 0x80);
    ad9649_spi_write(0xff, 0x01);
    ad9649_spi_write(0xff, 0x00);
    xil_printf("AD9649[0x016]: %02x\n\r", ad9649_spi_read(0x16));
    delay_ms(10);
    if (adc_delay()) {
      xil_printf("adc_setup: can not set a zero error delay!\n\r");
      adc_delay_1(0);
    }
  }
}

// ***************************************************************************
// ***************************************************************************

int main() {

  u32 mode;

  init_platform();

  Xil_Out32((CF_BASEADDR + 0x24), 0x0);
  Xil_Out32((CF_BASEADDR + 0x28), 0x0);

  xil_printf("AD9649[0x000]: %02x\n\r", ad9649_spi_read(0x000));
  xil_printf("AD9649[0x001]: %02x\n\r", ad9649_spi_read(0x001));

  adc_setup(0);

  // data pattern checks

  for (mode = 0x01; mode <= 0x07; mode++) {
    adc_test(mode, 0x0);
    adc_test(mode, 0x1);
  }

  ad9649_spi_write(0x14, 0x00);
  ad9649_spi_write(0x0d, 0x00);
  ad9649_spi_write(0xff, 0x01);
  xil_printf("done\n\r");
  cleanup_platform();
  return 0;
}


// ***************************************************************************
// ***************************************************************************

